@@ -3,7 +3,6 @@
 import abc
 import csv
 import json
-import sys
 from dataclasses import dataclass, field
 from typing import (
     Any,
@@ -25,6 +24,7 @@ from morecantile import Tile, TileMatrixSet
 from morecantile import tms as default_tms
 from morecantile.defaults import TileMatrixSets
 from pygeofilter.ast import AstType
+from typing_extensions import Annotated
 
 from tipg import model
 from tipg.collections import Catalog, Collection
@@ -33,6 +33,7 @@ from tipg.dependencies import (
     CollectionParams,
     ItemsOutputType,
     OutputType,
+    QueryablesOutputType,
     TileParams,
     bbox_query,
     datetime_query,
@@ -56,12 +57,6 @@ from starlette.requests import Request
 from starlette.responses import HTMLResponse, Response, StreamingResponse
 from starlette.templating import Jinja2Templates, _TemplateResponse
 
-if sys.version_info >= (3, 9):
-    from typing import Annotated  # pylint: disable=no-name-in-module
-else:
-    from typing_extensions import Annotated
-
-
 tms_settings = TMSSettings()
 mvt_settings = MVTSettings()
 features_settings = FeaturesSettings()
@@ -72,6 +67,32 @@ DEFAULT_TEMPLATES = Jinja2Templates(
 )  # type:ignore
 
 
+COMMON_CONFORMS = [
+    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/core",
+    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/landingPage",
+    "http://www.opengis.net/spec/ogcapi-common-2/1.0/conf/collections",
+    "http://www.opengis.net/spec/ogcapi-common-2/1.0/conf/simple-query",
+    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/json",
+    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/html",
+    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/oas30",
+]
+FEATURES_CONFORMS = [
+    "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/core",
+    "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/html",
+    "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/oas30",
+    "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/geojson",
+    "http://www.opengis.net/spec/ogcapi-features-3/1.0/conf/filter",
+    "http://www.opengis.net/spec/ogcapi-features-3/1.0/conf/features-filter",
+]
+TILES_CONFORMS = [
+    "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/core",
+    "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/oas30",
+    "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/mvt",
+    "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/tileset",
+    "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/tilesets-list",
+]
+
+
 def create_csv_rows(data: Iterable[Dict]) -> Generator[str, None, None]:
     """Creates an iterator that returns lines of csv from an iterable of dicts."""
 
@@ -272,23 +293,12 @@ class EndpointsFactory(metaclass=abc.ABCMeta):
             output_type: Annotated[Optional[MediaType], Depends(OutputType)] = None,
         ):
             """Get conformance."""
-            data = model.Conformance(
-                conformsTo=[
-                    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/core",
-                    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/landingPage",
-                    "http://www.opengis.net/spec/ogcapi-common-2/1.0/conf/collections",
-                    "http://www.opengis.net/spec/ogcapi-common-2/1.0/conf/simple-query",
-                    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/json",
-                    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/html",
-                    "http://www.opengis.net/spec/ogcapi-common-1/1.0/conf/oas30",
-                    *self.conforms_to,
-                ]
-            )
+            data = model.Conformance(conformsTo=[*COMMON_CONFORMS, *self.conforms_to])
 
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="conformance",
                 )
 
@@ -348,7 +358,7 @@ class EndpointsFactory(metaclass=abc.ABCMeta):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="landing",
                 )
 
@@ -362,14 +372,7 @@ class OGCFeaturesFactory(EndpointsFactory):
     @property
     def conforms_to(self) -> List[str]:
         """Factory conformances."""
-        return [
-            "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/core",
-            "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/html",
-            "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/oas30",
-            "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/geojson",
-            "http://www.opengis.net/spec/ogcapi-features-3/1.0/conf/filter",
-            "http://www.opengis.net/spec/ogcapi-features-3/1.0/conf/features-filter",
-        ]
+        return FEATURES_CONFORMS
 
     def links(self, request: Request) -> List[model.Link]:
         """OGC Features API links."""
@@ -523,7 +526,7 @@ class OGCFeaturesFactory(EndpointsFactory):
                         rel="next",
                         type=MediaType.json,
                         title="Next page",
-                    ).dict(exclude_none=True),
+                    ),
                 )
 
             if offset:
@@ -545,7 +548,7 @@ class OGCFeaturesFactory(EndpointsFactory):
                         rel="prev",
                         type=MediaType.json,
                         title="Previous page",
-                    ).dict(exclude_none=True),
+                    ),
                 )
 
             data = model.Collections(
@@ -554,41 +557,39 @@ class OGCFeaturesFactory(EndpointsFactory):
                 numberReturned=items_returned,
                 collections=[
                     model.Collection(
-                        **{
-                            "id": collection.id,
-                            "title": collection.id,
-                            "description": collection.description,
-                            "extent": collection.extent,
-                            "links": [
-                                model.Link(
-                                    href=self.url_for(
-                                        request,
-                                        "collection",
-                                        collectionId=collection.id,
-                                    ),
-                                    rel="collection",
-                                    type=MediaType.json,
+                        id=collection.id,
+                        title=collection.id,
+                        description=collection.description,
+                        extent=collection.extent,
+                        links=[
+                            model.Link(
+                                href=self.url_for(
+                                    request,
+                                    "collection",
+                                    collectionId=collection.id,
                                 ),
-                                model.Link(
-                                    href=self.url_for(
-                                        request,
-                                        "items",
-                                        collectionId=collection.id,
-                                    ),
-                                    rel="items",
-                                    type=MediaType.geojson,
+                                rel="collection",
+                                type=MediaType.json,
+                            ),
+                            model.Link(
+                                href=self.url_for(
+                                    request,
+                                    "items",
+                                    collectionId=collection.id,
                                 ),
-                                model.Link(
-                                    href=self.url_for(
-                                        request,
-                                        "queryables",
-                                        collectionId=collection.id,
-                                    ),
-                                    rel="queryables",
-                                    type=MediaType.schemajson,
+                                rel="items",
+                                type=MediaType.geojson,
+                            ),
+                            model.Link(
+                                href=self.url_for(
+                                    request,
+                                    "queryables",
+                                    collectionId=collection.id,
                                 ),
-                            ],
-                        }
+                                rel="queryables",
+                                type=MediaType.schemajson,
+                            ),
+                        ],
                     )
                     for collection in collections_list
                 ],
@@ -597,7 +598,7 @@ class OGCFeaturesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="collections",
                 )
 
@@ -624,57 +625,57 @@ class OGCFeaturesFactory(EndpointsFactory):
         ):
             """Metadata for a feature collection."""
 
-            data = model.Collection(
-                **{
-                    **collection.dict(),
+            data = model.Collection.model_validate(
+                {
+                    **collection.model_dump(),
                     "title": collection.id,
                     "extent": collection.extent,
                     "links": [
-                        model.Link(
-                            href=self.url_for(
+                        {
+                            "href": self.url_for(
                                 request,
                                 "collection",
                                 collectionId=collection.id,
                             ),
-                            rel="self",
-                            type=MediaType.json,
-                        ),
-                        model.Link(
-                            title="Items",
-                            href=self.url_for(
+                            "rel": "self",
+                            "type": "application/json",
+                        },
+                        {
+                            "title": "Items",
+                            "href": self.url_for(
                                 request, "items", collectionId=collection.id
                             ),
-                            rel="items",
-                            type=MediaType.geojson,
-                        ),
-                        model.Link(
-                            title="Items (CSV)",
-                            href=self.url_for(
+                            "rel": "items",
+                            "type": "application/geo+json",
+                        },
+                        {
+                            "title": "Items (CSV)",
+                            "href": self.url_for(
                                 request, "items", collectionId=collection.id
                             )
                             + "?f=csv",
-                            rel="alternate",
-                            type=MediaType.csv,
-                        ),
-                        model.Link(
-                            title="Items (GeoJSONSeq)",
-                            href=self.url_for(
+                            "rel": "alternate",
+                            "type": "text/csv",
+                        },
+                        {
+                            "title": "Items (GeoJSONSeq)",
+                            "href": self.url_for(
                                 request, "items", collectionId=collection.id
                             )
                             + "?f=geojsonseq",
-                            rel="alternate",
-                            type=MediaType.geojsonseq,
-                        ),
-                        model.Link(
-                            title="Queryables",
-                            href=self.url_for(
+                            "rel": "alternate",
+                            "type": "application/geo+json-seq",
+                        },
+                        {
+                            "title": "Queryables",
+                            "href": self.url_for(
                                 request,
                                 "queryables",
                                 collectionId=collection.id,
                             ),
-                            rel="queryables",
-                            type=MediaType.schemajson,
-                        ),
+                            "rel": "queryables",
+                            "type": "application/schema+json",
+                        },
                     ],
                 }
             )
@@ -682,7 +683,7 @@ class OGCFeaturesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="collection",
                 )
 
@@ -706,29 +707,26 @@ class OGCFeaturesFactory(EndpointsFactory):
         def queryables(
             request: Request,
             collection: Annotated[Collection, Depends(self.collection_dependency)],
-            output_type: Annotated[Optional[MediaType], Depends(OutputType)] = None,
+            output_type: Annotated[
+                Optional[MediaType], Depends(QueryablesOutputType)
+            ] = None,
         ):
             """Queryables for a feature collection.
 
             ref: http://docs.ogc.org/DRAFTS/19-079r1.html#filter-queryables
             """
             qs = "?" + str(request.query_params) if request.query_params else ""
-
+            self_url = self.url_for(request, "queryables", collectionId=collection.id)
             data = model.Queryables(
-                **{
-                    "title": collection.id,
-                    "$id": self.url_for(
-                        request, "queryables", collectionId=collection.id
-                    )
-                    + qs,
-                    "properties": collection.queryables,
-                }
+                title=collection.id,
+                link=self_url + qs,
+                properties=collection.queryables,
             )
 
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="queryables",
                 )
 
@@ -895,21 +893,21 @@ class OGCFeaturesFactory(EndpointsFactory):
 
             qs = "?" + str(request.query_params) if request.query_params else ""
             links: List[Dict] = [
-                model.Link(
-                    title="Collection",
-                    href=self.url_for(
+                {
+                    "title": "Collection",
+                    "href": self.url_for(
                         request, "collection", collectionId=collection.id
                     ),
-                    rel="collection",
-                    type=MediaType.json,
-                ).dict(exclude_none=True),
-                model.Link(
-                    title="Items",
-                    href=self.url_for(request, "items", collectionId=collection.id)
+                    "rel": "collection",
+                    "type": "application/json",
+                },
+                {
+                    "title": "Items",
+                    "href": self.url_for(request, "items", collectionId=collection.id)
                     + qs,
-                    rel="self",
-                    type=MediaType.geojson,
-                ).dict(exclude_none=True),
+                    "rel": "self",
+                    "type": "application/geo+json",
+                },
             ]
 
             items_returned = len(items["features"])
@@ -924,12 +922,12 @@ class OGCFeaturesFactory(EndpointsFactory):
                     + f"?{query_params}"
                 )
                 links.append(
-                    model.Link(
-                        href=url,
-                        rel="next",
-                        type=MediaType.geojson,
-                        title="Next page",
-                    ).dict(exclude_none=True),
+                    {
+                        "href": url,
+                        "rel": "next",
+                        "type": "application/geo+json",
+                        "title": "Next page",
+                    },
                 )
 
             if offset:
@@ -946,12 +944,12 @@ class OGCFeaturesFactory(EndpointsFactory):
                     url += f"?{query_params}"
 
                 links.append(
-                    model.Link(
-                        href=url,
-                        rel="prev",
-                        type=MediaType.geojson,
-                        title="Previous page",
-                    ).dict(exclude_none=True),
+                    {
+                        "href": url,
+                        "rel": "prev",
+                        "type": "application/geo+json",
+                        "title": "Previous page",
+                    },
                 )
 
             data = {
@@ -968,27 +966,27 @@ class OGCFeaturesFactory(EndpointsFactory):
                     {
                         **feature,  # type: ignore
                         "links": [
-                            model.Link(
-                                title="Collection",
-                                href=self.url_for(
+                            {
+                                "title": "Collection",
+                                "href": self.url_for(
                                     request,
                                     "collection",
                                     collectionId=collection.id,
                                 ),
-                                rel="collection",
-                                type=MediaType.json,
-                            ).dict(exclude_none=True),
-                            model.Link(
-                                title="Item",
-                                href=self.url_for(
+                                "rel": "collection",
+                                "type": "application/json",
+                            },
+                            {
+                                "title": "Item",
+                                "href": self.url_for(
                                     request,
                                     "item",
                                     collectionId=collection.id,
                                     itemId=feature.get("id"),
                                 ),
-                                rel="item",
-                                type=MediaType.json,
-                            ).dict(exclude_none=True),
+                                "rel": "item",
+                                "type": "application/geo+json",
+                            },
                         ],
                     }
                     for feature in items["features"]
@@ -1153,23 +1151,23 @@ class OGCFeaturesFactory(EndpointsFactory):
             data = {
                 **feature,  # type: ignore
                 "links": [
-                    model.Link(
-                        href=self.url_for(
+                    {
+                        "href": self.url_for(
                             request, "collection", collectionId=collection.id
                         ),
-                        rel="collection",
-                        type=MediaType.json,
-                    ).dict(exclude_none=True),
-                    model.Link(
-                        href=self.url_for(
+                        "rel": "collection",
+                        "type": "application/json",
+                    },
+                    {
+                        "href": self.url_for(
                             request,
                             "item",
                             collectionId=collection.id,
                             itemId=itemId,
                         ),
-                        rel="self",
-                        type=MediaType.geojson,
-                    ).dict(exclude_none=True),
+                        "rel": "self",
+                        "type": "application/geo+json",
+                    },
                 ],
             }
 
@@ -1195,13 +1193,7 @@ class OGCTilesFactory(EndpointsFactory):
     @property
     def conforms_to(self) -> List[str]:
         """Factory conformances."""
-        return [
-            "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/core",
-            "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/oas30",
-            "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/mvt",
-            "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/tileset",
-            "http://www.opengis.net/spec/ogcapi-tiles-1/1.0/conf/tilesets-list",
-        ]
+        return TILES_CONFORMS
 
     def links(self, request: Request) -> List[model.Link]:
         """OGC Tiles API links."""
@@ -1288,21 +1280,21 @@ class OGCTilesFactory(EndpointsFactory):
             """
             data = model.TileMatrixSetList(
                 tileMatrixSets=[
-                    {
-                        "id": tms_id,
-                        "links": [
-                            {
-                                "href": self.url_for(
+                    model.TileMatrixSetRef(
+                        id=tms_id,
+                        title=f"Definition of {tms_id} tileMatrixSets",
+                        links=[
+                            model.TileMatrixSetLink(
+                                href=self.url_for(
                                     request,
                                     "tilematrixset",
                                     tileMatrixSetId=tms_id,
                                 ),
-                                "rel": "http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
-                                "type": "application/json",
-                                "title": f"Definition of {tms_id} tileMatrixSets",
-                            }
+                                rel="http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
+                                type=MediaType.json,
+                            )
                         ],
-                    }
+                    )
                     for tms_id in self.supported_tms.list()
                 ]
             )
@@ -1310,7 +1302,7 @@ class OGCTilesFactory(EndpointsFactory):
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="tilematrixsets",
                 )
 
@@ -1346,10 +1338,15 @@ class OGCTilesFactory(EndpointsFactory):
 
             if output_type == MediaType.html:
                 # For visualization purpose we add the tms bbox
-                data = {**data.dict(exclude_none=True), "bbox": data.bbox}
+                data = {
+                    **data.model_dump(exclude_none=True, mode="json"),
+                    "bbox": data.bbox,
+                }
                 return self._create_html_response(
                     request,
-                    json.dumps(data),
+                    json.dumps(
+                        data,
+                    ),
                     template_name="tilematrixset",
                 )
 
@@ -1360,7 +1357,14 @@ class OGCTilesFactory(EndpointsFactory):
             response_model=model.TileSetList,
             response_class=ORJSONResponse,
             response_model_exclude_none=True,
-            responses={200: {"content": {MediaType.json.value: {}}}},
+            responses={
+                200: {
+                    "content": {
+                        MediaType.json.value: {},
+                        MediaType.html.value: {},
+                    }
+                }
+            },
             summary="Retrieve a list of available vector tilesets for the specified collection.",
             operation_id=".collection.vector.getTileSetsList",
         )
@@ -1378,59 +1382,61 @@ class OGCTilesFactory(EndpointsFactory):
                     "crs": "http://www.opengis.net/def/crs/OGC/1.3/CRS84",
                 }
 
-            data = model.TileSetList(
-                tilesets=[
-                    model.TileSet(
-                        title=f"'{collection.id}' tileset tiled using {tms} TileMatrixSet",
-                        dataType="vector",
-                        crs=self.supported_tms.get(tms).crs,
-                        boundingBox=collection_bbox,
-                        links=[
-                            {
-                                "href": self.url_for(
-                                    request,
-                                    "collection_tileset",
-                                    collectionId=collection.id,
-                                    tileMatrixSetId=tms,
-                                ),
-                                "rel": "self",
-                                "type": MediaType.json,
-                                "title": f"'{collection.id}' tileset tiled using {tms} TileMatrixSet",
-                            },
-                            {
-                                "href": self.url_for(
-                                    request,
-                                    "tilematrixset",
-                                    tileMatrixSetId=tms,
-                                ),
-                                "rel": "http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
-                                "type": MediaType.json,
-                                "title": f"Definition of '{tms}' tileMatrixSet",
-                            },
-                            {
-                                "href": self.url_for(
-                                    request,
-                                    "collection_get_tile",
-                                    tileMatrixSetId=tms,
-                                    collectionId=collection.id,
-                                    z="{z}",
-                                    x="{x}",
-                                    y="{y}",
-                                ),
-                                "rel": "tile",
-                                "type": MediaType.mvt,
-                                "title": "Templated link for retrieving Vector tiles",
-                            },
-                        ],
-                    )
-                    for tms in self.supported_tms.list()
-                ]
+            data = model.TileSetList.model_validate(
+                {
+                    "tilesets": [
+                        {
+                            "title": f"'{collection.id}' tileset tiled using {tms} TileMatrixSet",
+                            "dataType": "vector",
+                            "crs": self.supported_tms.get(tms).crs,
+                            "boundingBox": collection_bbox,
+                            "links": [
+                                {
+                                    "href": self.url_for(
+                                        request,
+                                        "collection_tileset",
+                                        collectionId=collection.id,
+                                        tileMatrixSetId=tms,
+                                    ),
+                                    "rel": "self",
+                                    "type": "application/json",
+                                    "title": f"'{collection.id}' tileset tiled using {tms} TileMatrixSet",
+                                },
+                                {
+                                    "href": self.url_for(
+                                        request,
+                                        "tilematrixset",
+                                        tileMatrixSetId=tms,
+                                    ),
+                                    "rel": "http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
+                                    "type": "application/json",
+                                    "title": f"Definition of '{tms}' tileMatrixSet",
+                                },
+                                {
+                                    "href": self.url_for(
+                                        request,
+                                        "collection_get_tile",
+                                        tileMatrixSetId=tms,
+                                        collectionId=collection.id,
+                                        z="{z}",
+                                        x="{x}",
+                                        y="{y}",
+                                    ),
+                                    "rel": "tile",
+                                    "type": "application/vnd.mapbox-vector-tile",
+                                    "title": "Templated link for retrieving Vector tiles",
+                                },
+                            ],
+                        }
+                        for tms in self.supported_tms.list()
+                    ]
+                }
             )
 
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="tilesets",
                 )
 
@@ -1463,6 +1469,7 @@ class OGCTilesFactory(EndpointsFactory):
                     "upperRight": [bounds[2], bounds[3]],
                     "crs": "http://www.opengis.net/def/crs/OGC/1.3/CRS84",
                 }
+
                 tilematrix_limit = []
                 for matrix in tms:
                     ulTile = tms.tile(bounds[0], bounds[3], int(matrix.id))
@@ -1492,55 +1499,57 @@ class OGCTilesFactory(EndpointsFactory):
                     for matrix in tms
                 ]
 
-            data = model.TileSet(
-                title=f"'{collection.id}' tileset tiled using {tileMatrixSetId} TileMatrixSet",
-                dataType="vector",
-                crs=tms.crs,
-                boundingBox=collection_bbox,
-                links=[
-                    {
-                        "href": self.url_for(
-                            request,
-                            "collection_tileset",
-                            collectionId=collection.id,
-                            tileMatrixSetId=tileMatrixSetId,
-                        ),
-                        "rel": "self",
-                        "type": MediaType.json,
-                        "title": f"'{collection.id}' tileset tiled using {tileMatrixSetId} TileMatrixSet",
-                    },
-                    {
-                        "href": self.url_for(
-                            request,
-                            "tilematrixset",
-                            tileMatrixSetId=tileMatrixSetId,
-                        ),
-                        "rel": "http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
-                        "type": MediaType.json,
-                        "title": f"Definition of '{tileMatrixSetId}' tileMatrixSet",
-                    },
-                    {
-                        "href": self.url_for(
-                            request,
-                            "collection_get_tile",
-                            tileMatrixSetId=tileMatrixSetId,
-                            collectionId=collection.id,
-                            z="{z}",
-                            x="{x}",
-                            y="{y}",
-                        ),
-                        "rel": "tile",
-                        "type": MediaType.mvt,
-                        "title": "Templated link for retrieving Vector tiles",
-                    },
-                ],
-                tileMatrixSetLimits=tilematrix_limit,
+            data = model.TileSet.model_validate(
+                {
+                    "title": f"'{collection.id}' tileset tiled using {tileMatrixSetId} TileMatrixSet",
+                    "dataType": "vector",
+                    "crs": tms.crs,
+                    "boundingBox": collection_bbox,
+                    "links": [
+                        {
+                            "href": self.url_for(
+                                request,
+                                "collection_tileset",
+                                collectionId=collection.id,
+                                tileMatrixSetId=tileMatrixSetId,
+                            ),
+                            "rel": "self",
+                            "type": "application/json",
+                            "title": f"'{collection.id}' tileset tiled using {tileMatrixSetId} TileMatrixSet",
+                        },
+                        {
+                            "href": self.url_for(
+                                request,
+                                "tilematrixset",
+                                tileMatrixSetId=tileMatrixSetId,
+                            ),
+                            "rel": "http://www.opengis.net/def/rel/ogc/1.0/tiling-schemes",
+                            "type": "application/json",
+                            "title": f"Definition of '{tileMatrixSetId}' tileMatrixSet",
+                        },
+                        {
+                            "href": self.url_for(
+                                request,
+                                "collection_get_tile",
+                                tileMatrixSetId=tileMatrixSetId,
+                                collectionId=collection.id,
+                                z="{z}",
+                                x="{x}",
+                                y="{y}",
+                            ),
+                            "rel": "tile",
+                            "type": "application/vnd.mapbox-vector-tile",
+                            "title": "Templated link for retrieving Vector tiles",
+                        },
+                    ],
+                    "tileMatrixSetLimits": tilematrix_limit,
+                }
             )
 
             if output_type == MediaType.html:
                 return self._create_html_response(
                     request,
-                    data.json(exclude_none=True),
+                    data.model_dump_json(exclude_none=True),
                     template_name="tileset",
                 )
 
